<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/8/14 21:30
 */

namespace App\Http\Controllers\Backend\Server\System;

use App\Exceptions\ApiException;
use App\Http\Controllers\Controller;
use App\Http\Requests\Backend\Server\System\MenuRequest;
use App\Http\Services\Backend\Server\System\MenuService;
use Illuminate\Http\Client\Response;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;


class MenuController extends Controller
{
    private MenuService $menuService;

    function __construct(MenuService $menuService)
    {
        $this->menuService = $menuService;
//        $this->middleware('permission:system:menu:query|system:menu:create|system:menu:update|system:menu:delete', ['only' => ['index', 'show']]);
//        $this->middleware('permission:system:menu:create', ['only' => ['create', 'store']]);
//        $this->middleware('permission:system:menu:update', ['only' => ['edit', 'update']]);
//        $this->middleware('permission:system:menu:delete', ['only' => ['destroy']]);
    }

    public function index(Request $request): JsonResponse
    {
        // 处理首页逻辑
        $params = $request->all();
        $res = $this->menuService->getMenuList($params);
        return self::success($res);
    }

    public function show($id)
    {
        // 处理显示单个用户的逻辑
        $res = $this->menuService->getMenu($id);
        return self::success($res);
    }

    public function create()
    {
        // 显示创建用户的表单
    }

    /**
     * @throws ApiException
     */
    public function store(MenuRequest $request): JsonResponse
    {
        $params = $request->safe()->only(['name','type','sort','parentId','path','icon','component','componentName','status','visible','keepAlive','alwaysShow']);
        $permissions = $request->post('permission', '');
        $res = $this->menuService->createMenu($params, explode('|', $permissions));
        return self::success($res);
    }

    public function edit($id)
    {
        // 显示编辑用户的表单
    }

    /**
     * @throws ApiException
     */
    public function update(MenuRequest $request, int $id): JsonResponse
    {
        $params = [...$request->safe()->all(), 'id' => $id];
        $this->menuService->updateMenu($params);
        return self::success(true);
    }

    /**
     * @throws ApiException
     */
    public function destroy($id): JsonResponse
    {
        // 删除用户逻辑
        $res = $this->menuService->deleteMenu($id);
        return self::success($res);
    }

    public function simpleList()
    {
        $res = (new MenuService())->getSimpleMenuList();
        return self::success($res);
    }
}
