<?php
/**
 * @Name
 * @Description
 * @Author 刘学玺
 * @Date 2024/8/27 10:52
 */

namespace App\Http\Services\Backend\Server\System;

use App\Enums\Common\ErrorMessage;
use App\Http\Services\Service;
use App\Models\System\DictData;
use App\Models\System\DictType;
use Symfony\Component\HttpFoundation\Response;

class DictTypeService extends Service
{
    protected array $selectColumn = ['id', 'name', 'remark', 'status', 'type'];

    public function getDictType($id)
    {
        return DictType::query()->select($this->selectColumn)->find($id);
    }

    public function getDictTypeList($params): array
    {
        $dictType = DictType::query();
        !empty($params['name']) && $dictType->whereLike('name', $params['name']);
        !empty($params['type']) && $dictType->where('type', $params['type']);
        !empty($params['status']) && $dictType->where('status', $params['status']);
        !empty($params['createTime']) && $dictType->whereIn('created_at', $params['createTime']);
        $select = ['id', 'name', 'remark', 'status', 'type', 'creator', 'created_at'];
        $dictTypePage = DictType::query()->paginate($params['pageSize'], $select, 'page', $params['pageNo']);
        return ['list' => $dictTypePage->items(), 'total' => $dictTypePage->total()];
    }

    public function createDictType($data)
    {
        // 校验字典类型的唯一性
        $this->isExistType($data['name'], $data['type']);

        // 插入字典类型
        $dictType = self::toModel($data, DictType::class);
        return $dictType->create($dictType->getAttributes())->id;
    }

    public function updateDictType($data, $id)
    {
        empty($id) && self::error();
        // 校验字典类型的唯一性
        $this->isExistType($data['name'], $data['type'], $id);

        // 插入字典类型
        $dictType = self::toModel($data, DictType::class);
        $dictType->where('id', $id)->update($dictType->getAttributes());
    }

    public function deleteDictType(int $id)
    {
        $dictType = DictType::query()->find($id);
        is_null($dictType) && self::error('DICT_TYPE_NOT_EXISTS', Response::HTTP_UNPROCESSABLE_ENTITY);

        // 校验是否存在字典数据
        $this->validateDictDataExists($dictType->type);
        return $dictType->delete();
    }

    protected function isExistType($name, $type, $id = null)
    {
        $dict_id = DictType::query()->where('name', $name)->where('type', $type)->value('id');
        if (!empty($dict_id) && $dict_id !== $id) {
            // 写入日志
            self::error(ErrorMessage::DICT_TYPE_DUPLICATE, Response::HTTP_UNPROCESSABLE_ENTITY);
        }
    }

    protected function validateDictDataExists(string $dictType)
    {
        $isExists = DictData::query()->where('dict_type',$dictType)->count();
        $isExists && self::error(ErrorMessage::DICT_TYPE_EXISTS_DATA, Response::HTTP_UNPROCESSABLE_ENTITY);
    }

}
